from datetime import datetime
import base64
import os

from werkzeug import secure_filename

from .constants import DEFAULT_LANG, DEFAULT_PLATFORM
from .extensions import db
from .helpers import url_for


def clean_title(s):
    return secure_filename(s)[:64].strip('_').lower().replace('_', '-')


def generate_eid():
    return base64.urlsafe_b64encode(os.urandom(6))


class Question(db.Model):
    category_id = db.Column(db.String(32), primary_key=True)
    eid = db.Column(db.String(8), default=generate_eid, primary_key=True)
    lang = db.Column(db.String(8), default=DEFAULT_LANG, primary_key=True)
    platform = db.Column(db.String(32), default=DEFAULT_PLATFORM,
                         primary_key=True)

    title = db.Column(db.String(255), nullable=False)
    content = db.Column(db.Text, nullable=False)
    creator = db.Column(db.String(255), nullable=False)

    posted_at = db.Column(db.DateTime, default=datetime.utcnow)
    edited_at = db.Column(db.DateTime, nullable=True)

    position = db.Column(db.Integer, nullable=True)
    is_sticky = db.Column(db.Boolean, default=False)

    @classmethod
    def get_sticky(self, platform):
        return (Question.query
                .filter_by(is_sticky=True)
                .filter(Question.platform.in_([DEFAULT_PLATFORM, platform]))
                .order_by(Question.posted_at.asc())
                .all())

    def url(self, **kwargs):
        kwargs.setdefault('category_id', self.category_id)
        kwargs.setdefault('eid', self.eid)
        kwargs.setdefault('title', clean_title(self.title))
        if kwargs.get('lang') == DEFAULT_LANG:
            kwargs.pop('lang')
        return url_for('question', **kwargs)

    def json(self):
        return dict(
            category_id=self.category_id,
            eid=self.eid,
            lang=self.lang,
            platform=self.platform,
            title=self.title,
            content=self.content,
            creator=self.creator,
            is_sticky=self.is_sticky,
            )


class Category(db.Model):
    id = db.Column(db.String(32), primary_key=True)
    title = db.Column(db.String(32), nullable=False)

    @classmethod
    def create(cls, title):
        category = cls()
        category.id = clean_title(title)
        category.title = title
        return category

    def get_questions(self, platform):
        return (Question.query
                .filter_by(category_id=self.id)
                .filter(Question.platform.in_([DEFAULT_PLATFORM, platform]))
                .order_by(Question.posted_at.asc())
                .all())

    def url(self, **kwargs):
        kwargs.setdefault('category_id', self.id)
        return url_for('category', **kwargs)

    def json(self):
        return dict(id=self.id, title=self.title)



from flask_wtf import (Form, TextField, TextAreaField, SelectField,
                       BooleanField, Required)

from .constants import AVAILABLE_LANGS, AVAILABLE_PLATFORMS


class CategoryForm(Form):
    title = TextField("Title", validators=[Required()])


class QuestionForm(Form):
    category_id = SelectField("Category", validators=[Required()])
    lang = SelectField("Language", choices=AVAILABLE_LANGS,
                       validators=[Required()])
    platform = SelectField("Platform", choices=AVAILABLE_PLATFORMS,
                           validators=[Required()])

    title = TextField("Title", validators=[Required()])
    content = TextAreaField("Content", validators=[Required()])

    is_sticky = BooleanField("Sticky", default=False)



import base64
import os

from flask import (render_template, flash, abort, request, redirect, jsonify,
                   send_from_directory, g, url_for as _url_for)
from flask_googlelogin import UserMixin, login_user, logout_user, current_user
from werkzeug import secure_filename
import misaka as m

from .constants import (DEFAULT_LANG, DEFAULT_PLATFORM, AVAILABLE_PLATFORMS,
                        ALLOWED_EXTS)
from .extensions import login, db
from .forms import CategoryForm, QuestionForm
from .helpers import url_for
from .models import Category, Question


def init(app):

    @app.before_request
    def detect_platform():
        user_agent = request.headers.get('User-Agent').lower()
        if 'windows' in user_agent:
            g.detected_platform = 'win'
        elif 'macintosh' in user_agent:
            g.detected_platform = 'mac'
        elif 'linux' in user_agent:
            g.detected_platform = 'linux'

    @app.before_request
    def db_session_start():
        if request.endpoint and request.endpoint != 'static':
            db.session()


    @app.teardown_request
    def db_session_commit(exception=None):
        if request.endpoint and request.endpoint != 'static':
            if exception:
                db.session.rollback()
            else:
                db.session.commit()
            db.session.remove()


    @app.context_processor
    def common():
        params = request.view_args
        available_platforms = []
        for platform, platform_title in AVAILABLE_PLATFORMS:
            if platform == DEFAULT_PLATFORM:
                continue
            params['platform'] = platform
            available_platforms.append((platform, platform_title,
                                        _url_for(request.endpoint, **params)))
        return dict(
            m=m,
            url_for=url_for,
            current_user=current_user,
            selected_lang=request.args.get('lang', DEFAULT_LANG),
            selected_platform=request.args.get('platform', g.detected_platform),
            available_platforms=available_platforms,
            categories=Category.query.order_by(Category.title).all())


    @login.user_loader
    def get_user(user_id):
      user = UserMixin()
      user.id = user_id
      user.is_authenticated = lambda: user.id in app.config['ALLOWED_USERS']
      return user


    # Routes


    @app.route('/login')
    def login_redirect():
        return redirect(login.login_url())


    @app.route('/oauth2callback')
    @login.oauth2callback
    def oauth2callback(token, userinfo, **params):
        user = login.login_manager.user_callback(userinfo['id'])
        login_user(user)
        return redirect(url_for('index'))


    @app.route('/logout')
    def logout():
        logout_user()
        return redirect(url_for('index'))


    @app.route('/')
    def index():
        selected_platform = request.args.get('platform', g.detected_platform)
        questions = Question.get_sticky(platform=selected_platform)
        return render_template('paano/index.html', questions=questions)


    @app.route('/new_category', methods=['GET', 'POST'])
    def new_category():
        form = CategoryForm(prefix='paano-category')
        if current_user.is_authenticated() and form.validate_on_submit():
            category = Category.create(form.title.data)
            db.session.add(category)
            db.session.flush()
            flash("Category saved")
            return redirect(category.url())
        return render_template('paano/new_category.html', form=form)


    @app.route('/new_question', methods=['GET', 'POST'])
    def new_question():
        form_args = dict(prefix='paano-question')

        category_id = request.args.get('category_id')
        eid = request.args.get('eid')
        if category_id and eid:
            base_question = Question.query.filter_by(category_id=category_id,
                                                     eid=eid).first()
            if base_question.platform == DEFAULT_PLATFORM:
                abort(400)
            if not base_question:
                abort(404)
            form_args['obj'] = base_question

        form = QuestionForm(**form_args)

        form.category_id.choices = [(c.id, c.title) for c in
                                    Category.query.order_by(Category.title).all()]
        if current_user.is_authenticated() and form.validate_on_submit():
            question = Question()
            question.eid = eid
            question.creator = current_user.id
            form.populate_obj(question)
            db.session.add(question)
            db.session.flush()
            flash("Question saved")
            return redirect(question.url(platform=question.platform))
        else:
            if request.method == 'POST':
                print form.errors

        return render_template('paano/new_question.html', form=form)


    @app.route('/categories/<category_id>', methods=['GET', 'POST', 'DELETE'])
    def category(category_id):
        category = Category.query.get_or_404(category_id)
        if current_user.is_authenticated() and request.method == 'DELETE':
            Question.query.filter_by(category_id=category.id).delete()
            db.session.delete(category)
            return jsonify(success=True)
        selected_platform = request.args.get('platform', g.detected_platform)
        questions = category.get_questions(platform=selected_platform)
        form = CategoryForm(prefix='paano-category', obj=category)
        if current_user.is_authenticated():
            if request.args.get('edit'):
                return render_template('paano/category_form.html', form=form)
            if form.validate_on_submit():
                form.populate_obj(category)
                db.session.add(category)
                db.session.flush()
                flash("Category saved")
                return redirect(category.url())
        return render_template('paano/category.html', category=category,
                               questions=questions)


    @app.route('/categories/<category_id>/questions/<eid>/<title>',
               methods=['GET', 'POST', 'DELETE'])
    def question(category_id, eid, title):
        category = Category.query.get_or_404(category_id)

        selected_lang = request.args.get('lang', DEFAULT_LANG)
        selected_platform = request.args.get('platform', g.detected_platform)
        question = (Question.query
                    .filter_by(category_id=category_id, eid=eid,
                               lang=selected_lang)
                    .filter(Question.platform.in_([DEFAULT_PLATFORM,
                                                   selected_platform]))
                    .first())
        if not question:
            abort(404)

        if current_user.is_authenticated() and request.method == 'DELETE':
            db.session.delete(question)
            return jsonify(success=True)

        form = QuestionForm(prefix='paano-question', obj=question)
        form.category_id.choices = [(c.id, c.title) for c in
                                    Category.query.order_by(Category.title).all()]

        if current_user.is_authenticated():
            if request.args.get('edit'):
                return render_template('paano/question_form.html', form=form)
            if form.validate_on_submit():
                form.populate_obj(question)
                db.session.add(question)
                db.session.flush()
                flash("Question saved")
                return redirect(question.url(platform=question.platform))

        return render_template('paano/question.html', category=category,
                               question=question)


    @app.route('/upload', methods=['GET', 'POST'])
    def upload():
        if request.method == 'POST':
            f = request.files.get('userfile')
            if f:
                # Add a random prefix to keep filenames unique
                filename = (base64.urlsafe_b64encode(os.urandom(9)) + '_' +
                            secure_filename(f.filename))
                # Check if has valid extension
                if any(map(filename.endswith, ALLOWED_EXTS)):
                    # Store
                    f.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))
                    # Return upload path
                    return jsonify(filename=url_for('uploads', filename=filename))

        abort(400)


    @app.route('/uploads/<path:filename>')
    def uploads(filename):
        return send_from_directory(app.config['UPLOAD_FOLDER'], filename)



from flask import request, g, url_for as _url_for

from .constants import DEFAULT_LANG


def url_for(*args, **kwargs):
    """
    Override Flask's `url_for` to retain `lang` and `platform` params
    across requests
    """

    selected_lang = kwargs.get('lang', request.args.get('lang'))
    if selected_lang not in (None, DEFAULT_LANG):
        kwargs['lang'] = selected_lang

    selected_platform = kwargs.get('platform', request.args.get('platform'))
    if selected_platform == 'general':
        selected_platform = g.detected_platform
    if selected_platform not in (None, g.detected_platform):
        kwargs['platform'] = selected_platform

    return _url_for(*args, **kwargs)



"""
paano
=====

Dead simple FAQ/HowTo CMS
"""

from setuptools import setup


setup(
    name='paano',
    version='0.1.9',
    url='http://github.com/marksteve/paano',
    license='MIT',
    author='Mark Steve Samson',
    author_email='marksteve@insynchq.com',
    description='Dead simple FAQ/HowTo CMS',
    long_description=__doc__,
    packages=['paano'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'Flask==0.9',
        'Flask-Babel==0.8',
        'Flask-SQLAlchemy==0.16',
        'Flask-WTF==0.8.2',
        'Flask-GoogleLogin==0.1.4',
        'misaka==1.0.2',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
    ],
)



ALLOWED_EXTS = set(['.png', '.jpg', '.jpeg', '.gif'])
DEFAULT_LANG = 'en'
AVAILABLE_LANGS = [
    ('en', 'English'),
    ]
DEFAULT_PLATFORM = 'general'
AVAILABLE_PLATFORMS = [
    ('general', 'General'),
    ('win', 'Windows'),
    ('mac', 'Mac'),
    ('linux', 'Linux'),
    ]
